using System;
using System.Security.Cryptography;
using System.Text;
using System.IO;

class HybridBeispiel
{
    static void Main()
    {
        try
        {
            // Schritt 1: Daten mit AES verschlsseln

            // Erzeugen einer neuen RijndaelManaged-Instanz samt Schlssel und IV
            RijndaelManaged aesAlg = new RijndaelManaged();

            // Schlssel und IV in eigenen Variablen speichern
            // (dient nur der bersichtlichkeit)
            byte[] AESkey = aesAlg.Key;
            byte[] AESiv  = aesAlg.IV;

            // Der zu verschlsselnde Klartext
            string klartext = "Dieser Text ist streng geheim!";

            // Der Name der Zieldatei
            string aesZieldatei = "schluesseltext.txt";

            // Verschlsseln des Klartextes mit dem erzeugten Schlssel und IV
            AESVerschluessele(klartext, aesZieldatei, AESkey, AESiv);

            //
            // Schritt 2: AES-Schlssel und -IV mit RSA verschlsseln
            //

            // Erzeugen einer RSACryptoServiceProvider-Instanz samt Schlsselpaar
            RSACryptoServiceProvider rsaAlg = new RSACryptoServiceProvider();

            // Schlssel in eigenen Variablen speichern
            // (dient nur der bersichtlichkeit)
            RSAParameters RSApublicKey  = rsaAlg.ExportParameters(false);
            RSAParameters RSAprivateKey = rsaAlg.ExportParameters(true);

            // Verschlsselung des AES-Schlssels und -IVs mit dem ffentlichen
            // RSA-Schlssel
            byte[] verschluesselterAESkey = RSAVerschluessele(AESkey, RSApublicKey);
            byte[] verschluesselterAESiv  = RSAVerschluessele(AESiv, RSApublicKey);


            //
            // Jetzt knnen die verschlsselten Daten ausgetauscht werden, z.B. indem
            // sie in eine Datei geschrieben oder ber ein Netzwerk bertragen werden.
            //


            //
            // Schritt 3: AES-Schlssel und -IV mit RSA entschlsseln
            //

            // Entschlsselung des AES-Schlssels und -IVs mit dem privaten RSA-Schlssel 
            byte[] entschluesselterAESkey = RSAEntschluessele(verschluesselterAESkey, RSAprivateKey);
            byte[] entschluesselterAESiv  = RSAEntschluessele(verschluesselterAESiv, RSAprivateKey);

            //
            // Schritt 4: Daten mit entschlsselten AES-Schlssel und -IV
            // entschlsseln
            //

            // Entschlsseln der Daten aus der Datei mit dem soeben entschlsselten 
            // AES-Schlssel und -IV
            string entschluesselt = AESEntschluessele(aesZieldatei, entschluesselterAESkey, entschluesselterAESiv);

            // Ausgabe des Klartext und des entschlsselten Textes
            Console.WriteLine("Klartext:      {0}", klartext);
            Console.WriteLine("Entschlsselt: {0}", entschluesselt);

            // Vertrauliche Daten lschen
            aesAlg.Clear();
            rsaAlg.Clear();
        }
        catch (Exception e)
        {
            Console.WriteLine("Fehler: {0}", e.Message);
        }
    }

    // 
    // AESVerschluessele, AESEntschluessele, RSAVerschluessele und
    // RSAEntschluessele sind die gleichen Funktionen wie in Listing 1 bzw. 2:
    //
    
    public static void AESVerschluessele(String klartext, String dateiname, byte[] schluessel, byte[] iv)
    {
        try
        {
            // ffnen oder Anlegen der Datei
            FileStream fStream = File.Open(dateiname, FileMode.OpenOrCreate);

            // Erzeugen einer neuen RijndaelManaged-Instanz 
            RijndaelManaged aesAlg = new RijndaelManaged();

            // Erzeugen eines CryptoStream mit dem FileStream und dem bergebenen
            // Schlssel und IV
            CryptoStream cStream = new CryptoStream(fStream, aesAlg.CreateEncryptor(schluessel, iv), CryptoStreamMode.Write);

            // Erzeugen eines StreamWriter mit dem CryptoStream.
            StreamWriter sWriter = new StreamWriter(cStream);

            try
            {
                // Klartext, d.h. die zu verschlsselnden Daten, in den Stream schreiben
                sWriter.WriteLine(klartext);
            }
            catch (Exception e)
            {
                Console.WriteLine("Fehler beim Verschlsseln: {0}", e.Message);
            }
            finally
            {
                // Streams schlieen
                sWriter.Close();
                cStream.Close();
                fStream.Close();
            }

            // Vertrauliche Daten lschen
            aesAlg.Clear();
        }
        catch (Exception e)
        {
            Console.WriteLine("Fehler beim Verschlsseln: {0}", e.Message);
        }
    }


    public static string AESEntschluessele(String dateiname, byte[] schluessel, byte[] iv)
    {
        try
        {
            // ffnen der Datei
            FileStream fStream = File.Open(dateiname, FileMode.Open);

            // Erzeugen einer neuen RijndaelManaged-Instanz 
            RijndaelManaged aesAlg = new RijndaelManaged();

            // Erzeugen eines CryptoStream mit dem FileStream und dem bergebenen
            // Schlssel und IV
            CryptoStream cStream = new CryptoStream(fStream, aesAlg.CreateDecryptor(schluessel, iv), CryptoStreamMode.Read);

            // Erzeugen eines StreamReader mit dem CryptoStream.
            StreamReader sReader = new StreamReader(cStream);

            string entschluesselterText = null;

            try
            {
                // Entschlsselte Daten aus dem Stream lesen
                entschluesselterText = sReader.ReadLine();
            }
            catch (Exception e)
            {
                Console.WriteLine("Fehler beim Entschlsseln: {0}", e.Message);
            }
            finally
            {
                // Streams und Datei schlieen
                sReader.Close();
                cStream.Close();
                fStream.Close();
            }

            // Vertrauliche Daten lschen
            aesAlg.Clear();
 
            // entschlsselten Text zurckgeben
            return entschluesselterText;
        }
        catch (Exception e)
        {
            Console.WriteLine("Fehler beim Entschlsseln: {0}", e.Message);
            return null;
        }
    }
    
    static public byte[] RSAVerschluessele(byte[] klartext, RSAParameters schluessel)
    {
        try
        {     
            // Erzeugen einer neuen Instanz von RSACryptoServiceProvider
            RSACryptoServiceProvider rsaAlg = new RSACryptoServiceProvider();

            // Importieren des bergebenen ffentlichen Schlssels
            rsaAlg.ImportParameters(schluessel);

            // Verschlsseln der bergebenen Daten
            // Der Boolean-Wert besagt, das kein OAEP-Padding erfolgt
            byte[] schluesseltext = rsaAlg.Encrypt(klartext, false);

            // Vertrauliche Daten lschen
            rsaAlg.Clear();

            // Schlsseltext zurckgeben
            return schluesseltext;
        }
        catch (Exception e)
        {
            Console.WriteLine("Fehler beim Verschlsseln: {0}", e.Message);
            return null;
        }
    }

    static public byte[] RSAEntschluessele(byte[] schluesseltext, RSAParameters schluessel)
    {
        try
        {
            // Erzeugen einer neuen Instanz von RSACryptoServiceProvider
            RSACryptoServiceProvider rsaAlg = new RSACryptoServiceProvider();

            // Importieren des bergebenen privaten Schlssels
            rsaAlg.ImportParameters(schluessel);

            // Entschlsseln der bergebenen Daten
            // Der Boolean-Wert besagt, das kein OAEP-Padding erfolgt
            byte[] entschluesselt = rsaAlg.Decrypt(schluesseltext, false);

            // Vertrauliche Daten lschen
            rsaAlg.Clear();

            // Schlsseltext zurckgeben
            return entschluesselt;
        }
        catch (Exception e)
        {
            Console.WriteLine("Fehler beim Entschlsseln: {0}", e.Message);
            return null;
        }
    }

}